=== modified file 'NEWS'
--- NEWS	2006-09-22 00:24:07 +0000
+++ NEWS	2006-09-26 23:15:00 +0000
@@ -1,6 +1,36 @@
 IN DEVELOPMENT
 
   IMPROVEMENTS:
+    * ``bzr help commands`` output is now shorter (Aaron Bentley)
+
+  INTERNALS:
+
+    * ``TestCase.run_bzr``, ``run_bzr_captured``, and ``run_bzr_subprocess``
+      can take a ``working_dir='foo'`` parameter, which will change directory 
+      for the command. (John Arbash Meinel)
+
+  BUG FIXES:
+
+    * bzr branch/pull no longer complain about progress bar cleanup when
+      interrupted during fetch.  (Aaron Bentley, #54000)
+
+    * ``WorkingTree.set_parent_trees()`` uses the trees to directly write
+      the basis inventory, rather than going through the repository. This
+      allows us to have 1 inventory read, and 2 inventory writes when
+      committing a new tree. (John Arbash Meinel)
+
+bzr 0.11rc2  2006-09-27
+
+  BUG FIXES:
+
+    * Test suite hangs on windows fixed. (Andrew Bennets, Alexander Belchenko).
+    
+    * Commit performance regression fixed. (Aaron Bentley, Robert Collins, John
+      Arbash Meinel).
+
+bzr 0.11rc1  2006-09-25
+
+  IMPROVEMENTS:
 
     * Knit files now wait to create their contents until the first data is
       added. The old code used to create an empty .knit and a .kndx with just
@@ -42,6 +72,15 @@
 
   BUG FIXES:
 
+    * 'bzr inventory [FILE...]' allows restricting the file list to a
+      specific set of files. (John Arbash Meinel, #3631)
+
+    * Don't abort when annotating empty files (John Arbash Meinel, #56814)
+
+    * Add ``Stanza.to_unicode()`` which can be passed to another Stanza
+      when nesting stanzas. Also, add ``read_stanza_unicode`` to handle when
+      reading a nested Stanza. (John Arbash Meinel)
+
     * Transform._set_mode() needs to stat the right file. 
       (John Arbash Meinel, #56549)
 
@@ -84,6 +123,13 @@
       paths. This may not work perfectly on all platforms, but has the best
       chance of working in the common case. (John Arbash Meinel, #56816)
 
+    * When committing, only files that exist in working tree or basis tree
+      may be specified (Aaron Bentley, #50793)
+
+  PORTABILITY:
+
+    * Fixes to run on Python 2.5 (Brian M. Carlson, Martin Pool, Marien Zwart)
+
   INTERNALS:
 
     * TestCaseInTempDir now creates a separate directory for HOME, rather

=== modified file 'README'
--- README	2006-09-12 21:56:31 +0000
+++ README	2006-09-25 00:03:15 +0000
@@ -1,6 +1,6 @@
-*****************************
-Release notes for Bazaar 0.10
-*****************************
+**********************
+README for Bazaar 0.11
+**********************
 
 Bazaar is a distributed/decentralized version control system.
 
@@ -26,7 +26,7 @@
  * vibrant and active community
 
 For usage instructions see 
-http://bazaar.canonical.com/IntroductionToBzr
+http://doc.bazaar-vcs.org/
 or run "bzr help"
 
 For installation instructions see ./INSTALL

=== modified file 'bzr'
--- bzr	2006-09-13 01:52:31 +0000
+++ bzr	2006-09-25 00:03:15 +0000
@@ -74,7 +74,7 @@
 import bzrlib.commands
 import bzrlib.trace
 
-if bzrlib.version_info[:3] != (0, 11, 0):
+if bzrlib.version_info[:3] != (0, 12, 0):
     sys.stderr.write("bzr: WARNING: bzrlib version doesn't match the bzr program.\n"
             "This may indicate an installation problem.\n"
             "bzrlib from %s is version %r\n"

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2006-09-12 21:56:31 +0000
+++ bzrlib/__init__.py	2006-09-25 00:03:15 +0000
@@ -35,7 +35,7 @@
 # Python version 2.0 is (2, 0, 0, 'final', 0)."  Additionally we use a
 # releaselevel of 'dev' for unreleased under-development code.
 
-version_info = (0, 11, 0, 'dev', 0)
+version_info = (0, 12, 0, 'dev', 0)
 
 if version_info[3] == 'final':
     version_string = '%d.%d.%d' % version_info[:3]

=== modified file 'bzrlib/annotate.py'
--- bzrlib/annotate.py	2006-06-20 03:57:11 +0000
+++ bzrlib/annotate.py	2006-09-21 19:41:29 +0000
@@ -39,7 +39,10 @@
 
     prevanno=''
     annotation = list(_annotate_file(branch, rev_id, file_id))
-    max_origin_len = max(len(origin) for origin in set(x[1] for x in annotation))
+    if len(annotation) == 0:
+        max_origin_len = 0
+    else:
+        max_origin_len = max(len(origin) for origin in set(x[1] for x in annotation))
     for (revno_str, author, date_str, line_rev_id, text ) in annotation:
         if verbose:
             anno = '%5s %-*s %8s ' % (revno_str, max_origin_len, author, date_str)

=== modified file 'bzrlib/benchmarks/bench_sftp.py'
--- bzrlib/benchmarks/bench_sftp.py	2006-08-18 16:48:53 +0000
+++ bzrlib/benchmarks/bench_sftp.py	2006-09-22 03:29:29 +0000
@@ -21,7 +21,7 @@
     bzrdir,
     )
 from bzrlib.benchmarks import Benchmark
-from bzrlib.tests import test_sftp_transport
+from bzrlib.tests import test_sftp_transport, TestSkipped
 
 try:
     import paramiko

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-09-22 04:52:17 +0000
+++ bzrlib/builtins.py	2006-09-27 02:28:44 +0000
@@ -54,6 +54,7 @@
 from bzrlib.revisionspec import RevisionSpec
 from bzrlib.trace import mutter, note, log_error, warning, is_quiet, info
 from bzrlib.transport.local import LocalTransport
+import bzrlib.tree
 from bzrlib.workingtree import WorkingTree
 
 
@@ -361,26 +362,49 @@
     """Show inventory of the current working copy or a revision.
 
     It is possible to limit the output to a particular entry
-    type using the --kind option.  For example; --kind file.
+    type using the --kind option.  For example: --kind file.
+
+    It is also possible to restrict the list of files to a specific
+    set. For example: bzr inventory --show-ids this/file
     """
 
     takes_options = ['revision', 'show-ids', 'kind']
-    
+    takes_args = ['file*']
+
     @display_command
-    def run(self, revision=None, show_ids=False, kind=None):
+    def run(self, revision=None, show_ids=False, kind=None, file_list=None):
         if kind and kind not in ['file', 'directory', 'symlink']:
             raise BzrCommandError('invalid kind specified')
-        tree = WorkingTree.open_containing(u'.')[0]
-        if revision is None:
-            inv = tree.read_working_inventory()
-        else:
+
+        work_tree, file_list = tree_files(file_list)
+
+        if revision is not None:
             if len(revision) > 1:
                 raise BzrCommandError('bzr inventory --revision takes'
-                    ' exactly one revision identifier')
-            inv = tree.branch.repository.get_revision_inventory(
-                revision[0].in_history(tree.branch).rev_id)
-
-        for path, entry in inv.entries():
+                                      ' exactly one revision identifier')
+            revision_id = revision[0].in_history(work_tree.branch).rev_id
+            tree = work_tree.branch.repository.revision_tree(revision_id)
+                        
+            # We include work_tree as well as 'tree' here
+            # So that doing '-r 10 path/foo' will lookup whatever file
+            # exists now at 'path/foo' even if it has been renamed, as
+            # well as whatever files existed in revision 10 at path/foo
+            trees = [tree, work_tree]
+        else:
+            tree = work_tree
+            trees = [tree]
+
+        if file_list is not None:
+            file_ids = bzrlib.tree.find_ids_across_trees(file_list, trees,
+                                                      require_versioned=True)
+            # find_ids_across_trees may include some paths that don't
+            # exist in 'tree'.
+            entries = sorted((tree.id2path(file_id), tree.inventory[file_id])
+                             for file_id in file_ids if file_id in tree)
+        else:
+            entries = tree.inventory.entries()
+
+        for path, entry in entries:
             if kind and kind != entry.kind:
                 continue
             if show_ids:
@@ -2004,6 +2028,7 @@
                 test_suite_factory = benchmarks.test_suite
                 if verbose is None:
                     verbose = True
+                # TODO: should possibly lock the history file...
                 benchfile = open(".perf_history", "at")
             else:
                 test_suite_factory = None

=== modified file 'bzrlib/cmd_version_info.py'
--- bzrlib/cmd_version_info.py	2006-09-21 16:33:39 +0000
+++ bzrlib/cmd_version_info.py	2006-09-22 15:19:55 +0000
@@ -18,11 +18,11 @@
 
 from bzrlib import (
     branch,
+    errors,
     version_info_formats,
     workingtree,
     )
 from bzrlib.commands import Command
-from bzrlib.errors import BzrCommandError
 from bzrlib.option import Option
 
 
@@ -36,9 +36,9 @@
         return version_info_formats.get_builder(format)
     except KeyError:
         formats = version_info_formats.get_builder_formats()
-        raise BzrCommandError('No known version info format %s.'
-                              ' Supported types are: %s'
-                              % (format, formats))
+        raise errors.BzrCommandError('No known version info format %s.'
+                                     ' Supported types are: %s'
+                                     % (format, formats))
 
 
 class cmd_version_info(Command):
@@ -70,7 +70,7 @@
         wt = None
         try:
             wt = workingtree.WorkingTree.open_containing(location)[0]
-        except NoWorkingTree:
+        except errors.NoWorkingTree:
             b = branch.Branch.open(location)
         else:
             b = wt.branch

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2006-09-08 18:46:29 +0000
+++ bzrlib/commands.py	2006-09-22 00:17:08 +0000
@@ -611,7 +611,7 @@
         return run_bzr(argv)
         # do this here inside the exception wrappers to catch EPIPE
         sys.stdout.flush()
-    except Exception, e:
+    except (KeyboardInterrupt, Exception), e:
         # used to handle AssertionError and KeyboardInterrupt
         # specially here, but hopefully they're handled ok by the logger now
         bzrlib.trace.report_exception(sys.exc_info(), sys.stderr)

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2006-09-22 04:52:17 +0000
+++ bzrlib/commit.py	2006-09-27 02:28:44 +0000
@@ -71,8 +71,11 @@
 
 from cStringIO import StringIO
 
+from bzrlib import (
+    errors,
+    tree,
+    )
 import bzrlib.config
-import bzrlib.errors as errors
 from bzrlib.errors import (BzrError, PointlessCommit,
                            ConflictsInTree,
                            StrictCommitFailed
@@ -279,6 +282,11 @@
             self.work_inv = self.work_tree.inventory
             self.basis_tree = self.work_tree.basis_tree()
             self.basis_inv = self.basis_tree.inventory
+            if specific_files is not None:
+                # Ensure specified files are versioned
+                # (We don't actually need the ids here)
+                tree.find_ids_across_trees(specific_files, 
+                                           [self.basis_tree, self.work_tree])
             # one to finish, one for rev and inventory, and one for each
             # inventory entry, and the same for the new inventory.
             # note that this estimate is too long when we do a partial tree
@@ -326,11 +334,8 @@
             # and now do the commit locally.
             self.branch.append_revision(self.rev_id)
 
-            # if the builder gave us the revisiontree it created back, we
-            # could use it straight away here.
-            # TODO: implement this.
-            self.work_tree.set_parent_trees([(self.rev_id,
-                self.branch.repository.revision_tree(self.rev_id))])
+            rev_tree = self.builder.revision_tree()
+            self.work_tree.set_parent_trees([(self.rev_id, rev_tree)])
             # now the work tree is up to date with the branch
             
             self.reporter.completed(self.branch.revno(), self.rev_id)
@@ -347,6 +352,40 @@
             self._cleanup()
         return self.rev_id
 
+    def _any_real_changes(self):
+        """Are there real changes between new_inventory and basis?
+
+        For trees without rich roots, inv.root.revision changes every commit.
+        But if that is the only change, we want to treat it as though there
+        are *no* changes.
+        """
+        new_entries = self.builder.new_inventory.iter_entries()
+        basis_entries = self.basis_inv.iter_entries()
+        new_path, new_root_ie = new_entries.next()
+        basis_path, basis_root_ie = basis_entries.next()
+
+        # This is a copy of InventoryEntry.__eq__ only leaving out .revision
+        def ie_equal_no_revision(this, other):
+            return ((this.file_id == other.file_id)
+                    and (this.name == other.name)
+                    and (this.symlink_target == other.symlink_target)
+                    and (this.text_sha1 == other.text_sha1)
+                    and (this.text_size == other.text_size)
+                    and (this.text_id == other.text_id)
+                    and (this.parent_id == other.parent_id)
+                    and (this.kind == other.kind)
+                    and (this.executable == other.executable)
+                    )
+        if not ie_equal_no_revision(new_root_ie, basis_root_ie):
+            return True
+
+        for new_ie, basis_ie in zip(new_entries, basis_entries):
+            if new_ie != basis_ie:
+                return True
+
+        # No actual changes present
+        return False
+
     def _check_pointless(self):
         if self.allow_pointless:
             return
@@ -357,10 +396,13 @@
         # basis
         if len(self.basis_inv) == 0 and len(self.builder.new_inventory) == 1:
             raise PointlessCommit()
+        # Shortcut, if the number of entries changes, then we obviously have
+        # a change
         if len(self.builder.new_inventory) != len(self.basis_inv):
             return
-        if (len(self.builder.new_inventory) != 1 and
-            self.builder.new_inventory != self.basis_inv):
+        # If length == 1, then we only have the root entry. Which means
+        # that there is no real difference (only the root could be different)
+        if (len(self.builder.new_inventory) != 1 and self._any_real_changes()):
             return
         raise PointlessCommit()
 

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-09-22 04:52:17 +0000
+++ bzrlib/errors.py	2006-09-27 02:28:44 +0000
@@ -44,7 +44,7 @@
 >>> try:
 ...   raise NotBranchError(path='/foo/bar')
 ... except:
-...   print sys.exc_type
+...   print '%s.%s' % (sys.exc_type.__module__, sys.exc_type.__name__)
 ...   print sys.exc_value
 ...   path = getattr(sys.exc_value, 'path', None)
 ...   if path is not None:
@@ -96,7 +96,7 @@
 class BzrError(StandardError):
     
     is_user_error = True
-    
+
     def __str__(self):
         # XXX: Should we show the exception class in 
         # exceptions that don't provide their own message?  
@@ -120,7 +120,11 @@
     # base classes should override the docstring with their human-
     # readable explanation
 
-    def __init__(self, **kwds):
+    def __init__(self, *args, **kwds):
+        # XXX: Use the underlying BzrError to always generate the args attribute
+        # if it doesn't exist.  We can't use super here, because exceptions are
+        # old-style classes in python2.4 (but new in 2.5).  --bmc, 20060426
+        BzrError.__init__(self, *args)
         for key, value in kwds.items():
             setattr(self, key, value)
 
@@ -283,8 +287,7 @@
 
     def __init__(self, msg, base, args):
         PathError.__init__(self, base, msg)
-        self.args = [base]
-        self.args.extend(args)
+        self.args = [base] + list(args)
 
 
 class UnsupportedProtocol(PathError):

=== modified file 'bzrlib/export/zip_exporter.py'
--- bzrlib/export/zip_exporter.py	2006-09-20 22:54:18 +0000
+++ bzrlib/export/zip_exporter.py	2006-09-23 19:05:44 +0000
@@ -18,8 +18,22 @@
 """
 
 import os
+import stat
+import zipfile
+
+from bzrlib import (
+    osutils,
+    )
 from bzrlib.trace import mutter
-import zipfile
+
+
+# Windows expects this bit to be set in the 'external_attr' section
+# Or it won't consider the entry a directory
+ZIP_DIRECTORY_BIT = (1 << 4)
+
+_FILE_ATTR = stat.S_IFREG
+_DIR_ATTR = stat.S_IFDIR | ZIP_DIRECTORY_BIT
+
 
 def zip_exporter(tree, dest, root):
     """ Export this tree to a new zip file.
@@ -49,24 +63,32 @@
             file_id = ie.file_id
             mutter("  export {%s} kind %s to %s", file_id, ie.kind, dest)
 
-            filename = os.path.join(root, dp).encode('utf8')
-            if ie.kind == "file": 
+            # zipfile.ZipFile switches all paths to forward
+            # slashes anyway, so just stick with that.
+            filename = osutils.pathjoin(root, dp).encode('utf8')
+            if ie.kind == "file":
                 zinfo = zipfile.ZipInfo(
                             filename=filename,
                             date_time=now)
                 zinfo.compress_type = compression
+                zinfo.external_attr = _FILE_ATTR
                 zipf.writestr(zinfo, tree.get_file_text(file_id))
             elif ie.kind == "directory":
+                # Directories must contain a trailing slash, to indicate
+                # to the zip routine that they are really directories and
+                # not just empty files.
                 zinfo = zipfile.ZipInfo(
-                            filename=filename,
+                            filename=filename + '/',
                             date_time=now)
                 zinfo.compress_type = compression
+                zinfo.external_attr = _DIR_ATTR
                 zipf.writestr(zinfo,'')
             elif ie.kind == "symlink":
                 zinfo = zipfile.ZipInfo(
                             filename=(filename + '.lnk'),
                             date_time=now)
                 zinfo.compress_type = compression
+                zinfo.external_attr = _FILE_ATTR
                 zipf.writestr(zinfo, ie.symlink_target)
 
         zipf.close()

=== modified file 'bzrlib/help.py'
--- bzrlib/help.py	2006-09-08 18:46:29 +0000
+++ bzrlib/help.py	2006-09-25 21:54:28 +0000
@@ -21,6 +21,7 @@
 
 # TODO: `help commands --all` should show hidden commands
 import textwrap
+from bzrlib import osutils 
 
 global_help = \
 """Bazaar -- a free distributed version-control tool
@@ -141,27 +142,29 @@
     from bzrlib.commands import (builtin_command_names,
                                  plugin_command_names,
                                  get_cmd_object)
-
     if outfile is None:
         outfile = sys.stdout
-
-    names = set()                       # to eliminate duplicates
-    names.update(builtin_command_names())
+    names = set(builtin_command_names()) # to eliminate duplicates
     names.update(plugin_command_names())
-    names = list(names)
-    names.sort()
-
-    for cmd_name in names:
-        cmd_object = get_cmd_object(cmd_name)
-        if cmd_object.hidden:
-            continue
-        print >>outfile, command_usage(cmd_object)
-
+    commands = ((n, get_cmd_object(n)) for n in names)
+    shown_commands = [(n, o) for n, o in commands if not o.hidden]
+    max_name = max(len(n) for n, o in shown_commands)
+    indent = ' ' * (max_name + 1)
+    width = osutils.terminal_width() - 1
+    for cmd_name, cmd_object in sorted(shown_commands):
         plugin_name = cmd_object.plugin_name()
-        print_command_plugin(cmd_object, outfile, '        %s\n')
+        if plugin_name is None:
+            plugin_name = ''
+        else:
+            plugin_name = ' [%s]' % plugin_name
 
         cmd_help = cmd_object.help()
         if cmd_help:
             firstline = cmd_help.split('\n', 1)[0]
-            print >>outfile, '        ' + firstline
-        
+        else:
+            firstline = ''
+        helpstring = '%-*s %s%s' % (max_name, cmd_name, firstline, plugin_name)
+        lines = textwrap.wrap(helpstring, subsequent_indent=indent,
+                              width=width)
+        for line in lines:
+            outfile.write(line + '\n')

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2006-09-08 18:52:17 +0000
+++ bzrlib/knit.py	2006-09-25 14:52:38 +0000
@@ -77,6 +77,7 @@
 from bzrlib import (
     cache_utf8,
     errors,
+    progress,
     )
 from bzrlib.errors import FileExists, NoSuchFile, KnitError, \
         InvalidRevisionId, KnitCorrupt, KnitHeaderError, \
@@ -798,10 +799,13 @@
             text_map[version_id] = text 
         return text_map, final_content 
 
-    def iter_lines_added_or_present_in_versions(self, version_ids=None):
+    def iter_lines_added_or_present_in_versions(self, version_ids=None, 
+                                                pb=None):
         """See VersionedFile.iter_lines_added_or_present_in_versions()."""
         if version_ids is None:
             version_ids = self.versions()
+        if pb is None:
+            pb = progress.DummyProgress()
         # we don't care about inclusions, the caller cares.
         # but we need to setup a list of records to visit.
         # we need version_id, position, length
@@ -819,33 +823,26 @@
                 data_pos, length = self._index.get_position(version_id)
                 version_id_records.append((version_id, data_pos, length))
 
-        pb = bzrlib.ui.ui_factory.nested_progress_bar()
         count = 0
         total = len(version_id_records)
-        try:
+        pb.update('Walking content.', count, total)
+        for version_id, data, sha_value in \
+            self._data.read_records_iter(version_id_records):
             pb.update('Walking content.', count, total)
-            for version_id, data, sha_value in \
-                self._data.read_records_iter(version_id_records):
-                pb.update('Walking content.', count, total)
-                method = self._index.get_method(version_id)
-                version_idx = self._index.lookup(version_id)
-                assert method in ('fulltext', 'line-delta')
-                if method == 'fulltext':
-                    content = self.factory.parse_fulltext(data, version_idx)
-                    for line in content.text():
+            method = self._index.get_method(version_id)
+            version_idx = self._index.lookup(version_id)
+            assert method in ('fulltext', 'line-delta')
+            if method == 'fulltext':
+                content = self.factory.parse_fulltext(data, version_idx)
+                for line in content.text():
+                    yield line
+            else:
+                delta = self.factory.parse_line_delta(data, version_idx)
+                for start, end, count, lines in delta:
+                    for origin, line in lines:
                         yield line
-                else:
-                    delta = self.factory.parse_line_delta(data, version_idx)
-                    for start, end, count, lines in delta:
-                        for origin, line in lines:
-                            yield line
-                count +=1
-            pb.update('Walking content.', total, total)
-            pb.finished()
-        except:
-            pb.update('Walking content.', total, total)
-            pb.finished()
-            raise
+            count +=1
+        pb.update('Walking content.', total, total)
         
     def num_versions(self):
         """See VersionedFile.num_versions()."""

=== modified file 'bzrlib/plugins/launchpad/test_register.py'
--- bzrlib/plugins/launchpad/test_register.py	2006-05-16 02:31:06 +0000
+++ bzrlib/plugins/launchpad/test_register.py	2006-09-21 23:06:54 +0000
@@ -71,6 +71,9 @@
 
 class InstrumentedXMLRPCTransport(xmlrpclib.Transport):
 
+    # Python 2.5's xmlrpclib looks for this.
+    _use_datetime = False
+
     def __init__(self, testcase):
         self.testcase = testcase
 

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-09-22 04:52:17 +0000
+++ bzrlib/repository.py	2006-09-27 02:28:44 +0000
@@ -426,20 +426,25 @@
         # revisions. We don't need to see all lines in the inventory because
         # only those added in an inventory in rev X can contain a revision=X
         # line.
-        for line in w.iter_lines_added_or_present_in_versions(selected_revision_ids):
-            start = line.find('file_id="')+9
-            if start < 9: continue
-            end = line.find('"', start)
-            assert end>= 0
-            file_id = _unescape_xml(line[start:end])
+        pb = ui.ui_factory.nested_progress_bar()
+        try:
+            for line in w.iter_lines_added_or_present_in_versions(
+                selected_revision_ids, pb=pb):
+                start = line.find('file_id="')+9
+                if start < 9: continue
+                end = line.find('"', start)
+                assert end>= 0
+                file_id = _unescape_xml(line[start:end])
 
-            start = line.find('revision="')+10
-            if start < 10: continue
-            end = line.find('"', start)
-            assert end>= 0
-            revision_id = _unescape_xml(line[start:end])
-            if revision_id in selected_revision_ids:
-                result.setdefault(file_id, set()).add(revision_id)
+                start = line.find('revision="')+10
+                if start < 10: continue
+                end = line.find('"', start)
+                assert end>= 0
+                revision_id = _unescape_xml(line[start:end])
+                if revision_id in selected_revision_ids:
+                    result.setdefault(file_id, set()).add(revision_id)
+        finally:
+            pb.finished()
         return result
 
     @needs_read_lock
@@ -2306,6 +2311,18 @@
             self.new_inventory, self._config)
         return self._new_revision_id
 
+    def revision_tree(self):
+        """Return the tree that was just committed.
+
+        After calling commit() this can be called to get a RevisionTree
+        representing the newly committed tree. This is preferred to
+        calling Repository.revision_tree() because that may require
+        deserializing the inventory, while we already have a copy in
+        memory.
+        """
+        return RevisionTree(self.repository, self.new_inventory,
+                            self._new_revision_id)
+
     def finish_inventory(self):
         """Tell the builder that the inventory is finished."""
         if self.new_inventory.root is None:
@@ -2367,10 +2384,9 @@
 
         # In this revision format, root entries have no knit or weave
         if ie is self.new_inventory.root:
-            if len(parent_invs) > 0 and parent_invs[0].root is not None:
-                ie.revision = parent_invs[0].root.revision
-            else:
-                ie.revision = None
+            # When serializing out to disk and back in
+            # root.revision is always _new_revision_id
+            ie.revision = self._new_revision_id
             return
         previous_entries = ie.find_previous_heads(
             parent_invs,

=== modified file 'bzrlib/rio.py'
--- bzrlib/rio.py	2006-09-08 18:46:29 +0000
+++ bzrlib/rio.py	2006-09-22 19:37:57 +0000
@@ -171,6 +171,28 @@
         """Return stanza as a single string"""
         return ''.join(self.to_lines())
 
+    def to_unicode(self):
+        """Return stanza as a single Unicode string.
+
+        This is most useful when adding a Stanza to a parent Stanza
+        """
+        if not self.items:
+            return u''
+
+        result = []
+        for tag, value in self.items:
+            if value == '':
+                result.append(tag + ': \n')
+            elif '\n' in value:
+                # don't want splitlines behaviour on empty lines
+                val_lines = value.split('\n')
+                result.append(tag + ': ' + val_lines[0] + '\n')
+                for line in val_lines[1:]:
+                    result.append('\t' + line + '\n')
+            else:
+                result.append(tag + ': ' + value + '\n')
+        return u''.join(result)
+
     def write(self, to_file):
         """Write stanza to a file"""
         to_file.writelines(self.to_lines())
@@ -223,17 +245,41 @@
 
     The raw lines must be in utf-8 encoding.
     """
-    items = []
+    unicode_iter = (line.decode('utf-8') for line in line_iter)
+    return read_stanza_unicode(unicode_iter)
+
+
+def read_stanza_unicode(unicode_iter):
+    """Read a Stanza from a list of lines or a file.
+
+    The lines should already be in unicode form. This returns a single
+    stanza that was read. If there is a blank line at the end of the Stanza,
+    it is consumed. It is not an error for there to be no blank line at
+    the end of the iterable. If there is a blank line at the beginning,
+    this is treated as an empty Stanza and None is returned.
+
+    Only the stanza lines and the trailing blank (if any) are consumed
+    from the unicode_iter
+
+    :param unicode_iter: A iterable, yeilding Unicode strings. See read_stanza
+        if you have a utf-8 encoded string.
+    :return: A Stanza object if there are any lines in the file.
+        None otherwise
+    """
     stanza = Stanza()
     tag = None
     accum_value = None
-    for line in line_iter:
+    
+    # TODO: jam 20060922 This code should raise real errors rather than
+    #       using 'assert' to process user input, or raising ValueError
+    #       rather than a more specific error.
+
+    for line in unicode_iter:
         if line is None or line == '':
             break       # end of file
         if line == '\n':
             break       # end of stanza
-        line = line.decode('utf-8')
-        assert line[-1] == '\n'
+        assert line.endswith('\n')
         real_l = line
         if line[0] == '\t': # continues previous value
             if tag is None:
@@ -245,11 +291,13 @@
             try:
                 colon_index = line.index(': ')
             except ValueError:
-                raise ValueError('tag/value separator not found in line %r' % real_l)
+                raise ValueError('tag/value separator not found in line %r'
+                                 % real_l)
             tag = str(line[:colon_index])
             assert valid_tag(tag), \
                     "invalid rio tag %r" % tag
             accum_value = line[colon_index+2:-1]
+
     if tag is not None: # add last tag-value
         stanza.add(tag, accum_value)
         return stanza

=== modified file 'bzrlib/symbol_versioning.py'
--- bzrlib/symbol_versioning.py	2006-09-05 21:35:51 +0000
+++ bzrlib/symbol_versioning.py	2006-09-25 01:10:49 +0000
@@ -30,6 +30,7 @@
            'zero_nine',
            'zero_ten',
            'zero_eleven',
+           'zero_twelve',
            ]
 
 from warnings import warn
@@ -41,6 +42,7 @@
 zero_nine = "%s was deprecated in version 0.9."
 zero_ten = "%s was deprecated in version 0.10."
 zero_eleven = "%s was deprecated in version 0.11."
+zero_twelve = "%s was deprecated in version 0.12."
 
 
 def set_warning_method(method):

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-09-22 04:52:17 +0000
+++ bzrlib/tests/__init__.py	2006-09-27 02:28:44 +0000
@@ -242,9 +242,11 @@
         if isinstance(err[1], TestSkipped):
             return self.addSkipped(test, err)    
         unittest.TestResult.addError(self, test, err)
-        method = getattr(test, 'setKeepLogfile', None)
-        if method is not None:
-            method()
+        # We can only do this if we have one of our TestCases, not if
+        # we have a doctest.
+        setKeepLogfile = getattr(test, 'setKeepLogfile', None)
+        if setKeepLogfile is not None:
+            setKeepLogfile()
         self.extractBenchmarkTime(test)
         if self.showAll:
             self.stream.writeln("ERROR %s" % self._testTimeString())
@@ -261,9 +263,11 @@
 
     def addFailure(self, test, err):
         unittest.TestResult.addFailure(self, test, err)
-        method = getattr(test, 'setKeepLogfile', None)
-        if method is not None:
-            method()
+        # We can only do this if we have one of our TestCases, not if
+        # we have a doctest.
+        setKeepLogfile = getattr(test, 'setKeepLogfile', None)
+        if setKeepLogfile is not None:
+            setKeepLogfile()
         self.extractBenchmarkTime(test)
         if self.showAll:
             self.stream.writeln(" FAIL %s" % self._testTimeString())
@@ -784,7 +788,8 @@
         """Shortcut that splits cmd into words, runs, and returns stdout"""
         return self.run_bzr_captured(cmd.split(), retcode=retcode)[0]
 
-    def run_bzr_captured(self, argv, retcode=0, encoding=None, stdin=None):
+    def run_bzr_captured(self, argv, retcode=0, encoding=None, stdin=None,
+                         working_dir=None):
         """Invoke bzr and return (stdout, stderr).
 
         Useful for code that wants to check the contents of the
@@ -805,6 +810,7 @@
         :param retcode: expected return code, or None for don't-care.
         :param encoding: encoding for sys.stdout and sys.stderr
         :param stdin: A string to be used as stdin for the command.
+        :param working_dir: Change to this directory before running
         """
         if encoding is None:
             encoding = bzrlib.user_encoding
@@ -826,6 +832,12 @@
             stdout=stdout,
             stderr=stderr)
         bzrlib.ui.ui_factory.stdin = stdin
+
+        cwd = None
+        if working_dir is not None:
+            cwd = osutils.getcwd()
+            os.chdir(working_dir)
+
         try:
             result = self.apply_redirected(stdin, stdout, stderr,
                                            bzrlib.commands.run_bzr_catch_errors,
@@ -833,6 +845,8 @@
         finally:
             logger.removeHandler(handler)
             bzrlib.ui.ui_factory = old_ui_factory
+            if cwd is not None:
+                os.chdir(cwd)
 
         out = stdout.getvalue()
         err = stderr.getvalue()
@@ -859,7 +873,9 @@
         retcode = kwargs.pop('retcode', 0)
         encoding = kwargs.pop('encoding', None)
         stdin = kwargs.pop('stdin', None)
-        return self.run_bzr_captured(args, retcode=retcode, encoding=encoding, stdin=stdin)
+        working_dir = kwargs.pop('working_dir', None)
+        return self.run_bzr_captured(args, retcode=retcode, encoding=encoding,
+                                     stdin=stdin, working_dir=working_dir)
 
     def run_bzr_decode(self, *args, **kwargs):
         if 'encoding' in kwargs:
@@ -915,7 +931,9 @@
         :param universal_newlines: Convert CRLF => LF
         """
         env_changes = kwargs.get('env_changes', {})
-        process = self.start_bzr_subprocess(args, env_changes=env_changes)
+        working_dir = kwargs.get('working_dir', None)
+        process = self.start_bzr_subprocess(args, env_changes=env_changes,
+                                            working_dir=working_dir)
         # We distinguish between retcode=None and retcode not passed.
         supplied_retcode = kwargs.get('retcode', 0)
         return self.finish_bzr_subprocess(process, retcode=supplied_retcode,
@@ -923,7 +941,8 @@
             process_args=args)
 
     def start_bzr_subprocess(self, process_args, env_changes=None,
-                             skip_if_plan_to_signal=False):
+                             skip_if_plan_to_signal=False,
+                             working_dir=None):
         """Start bzr in a subprocess for testing.
 
         This starts a new Python interpreter and runs bzr in there.
@@ -961,6 +980,11 @@
 
         bzr_path = self.get_bzr_path()
 
+        cwd = None
+        if working_dir is not None:
+            cwd = osutils.getcwd()
+            os.chdir(working_dir)
+
         try:
             # win32 subprocess doesn't support preexec_fn
             # so we will avoid using it on all platforms, just to
@@ -970,6 +994,9 @@
                              stdin=PIPE, stdout=PIPE, stderr=PIPE)
         finally:
             restore_environment()
+            if cwd is not None:
+                os.chdir(cwd)
+
         return process
 
     def get_bzr_path(self):

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2006-09-20 14:51:03 +0000
+++ bzrlib/tests/blackbox/__init__.py	2006-09-23 06:39:46 +0000
@@ -60,6 +60,7 @@
                      'bzrlib.tests.blackbox.test_ignored',
                      'bzrlib.tests.blackbox.test_info',
                      'bzrlib.tests.blackbox.test_init',
+                     'bzrlib.tests.blackbox.test_inventory',
                      'bzrlib.tests.blackbox.test_locale',
                      'bzrlib.tests.blackbox.test_log',
                      'bzrlib.tests.blackbox.test_logformats',

=== modified file 'bzrlib/tests/blackbox/test_annotate.py'
--- bzrlib/tests/blackbox/test_annotate.py	2006-08-24 00:21:44 +0000
+++ bzrlib/tests/blackbox/test_annotate.py	2006-09-21 19:41:29 +0000
@@ -97,3 +97,13 @@
         self.assertEquals(out, '')
         self.assertEquals(err, 'bzr: ERROR: bzr annotate --revision takes'
                                ' exactly 1 argument\n')
+
+    def test_annotate_empty_file(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([('tree/empty', '')])
+        tree.add('empty')
+        tree.commit('add empty file')
+
+        os.chdir('tree')
+        out, err = self.run_bzr('annotate', 'empty')
+        self.assertEqual('', out)

=== modified file 'bzrlib/tests/blackbox/test_export.py'
--- bzrlib/tests/blackbox/test_export.py	2006-09-21 16:14:58 +0000
+++ bzrlib/tests/blackbox/test_export.py	2006-09-23 19:05:44 +0000
@@ -19,9 +19,14 @@
 """
 
 import os
+import stat
+import sys
 import tarfile
 import zipfile
 
+from bzrlib.export import (
+    zip_exporter,
+    )
 from bzrlib.tests import TestSkipped
 from bzrlib.tests.blackbox import ExternalBase
 
@@ -95,6 +100,36 @@
         self.assertEqual(['test/' + fname.encode('utf8')],
                          sorted(zfile.namelist()))
 
+    def test_zip_export_directories(self):
+        tree = self.make_branch_and_tree('zip')
+        self.build_tree(['zip/a', 'zip/b/', 'zip/b/c', 'zip/d/'])
+        tree.add(['a', 'b', 'b/c', 'd'])
+        tree.commit('init')
+
+        os.chdir('zip')
+        self.run_bzr('export', 'test.zip')
+        zfile = zipfile.ZipFile('test.zip')
+        names = sorted(zfile.namelist())
+
+        # even on win32, zipfile.ZipFile changes all names to use
+        # forward slashes
+        self.assertEqual(['test/a', 'test/b/', 'test/b/c', 'test/d/'], names)
+
+        file_attr = stat.S_IFREG
+        dir_attr = stat.S_IFDIR | zip_exporter.ZIP_DIRECTORY_BIT
+
+        a_info = zfile.getinfo(names[0])
+        self.assertEqual(file_attr, a_info.external_attr)
+
+        b_info = zfile.getinfo(names[1])
+        self.assertEqual(dir_attr, b_info.external_attr)
+
+        c_info = zfile.getinfo(names[2])
+        self.assertEqual(file_attr, c_info.external_attr)
+
+        d_info = zfile.getinfo(names[3])
+        self.assertEqual(dir_attr, d_info.external_attr)
+
     def test_dir_export(self):
         tree = self.make_branch_and_tree('dir')
         self.build_tree(['dir/a'])

=== added file 'bzrlib/tests/blackbox/test_inventory.py'
--- bzrlib/tests/blackbox/test_inventory.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_inventory.py	2006-09-22 21:39:54 +0000
@@ -0,0 +1,116 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Black-box tests for 'bzr inventory'."""
+
+import os
+
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestInventory(TestCaseWithTransport):
+
+    def setUp(self):
+        TestCaseWithTransport.setUp(self)
+
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['a', 'b/', 'b/c'])
+
+        tree.add(['a', 'b', 'b/c'], ['a-id', 'b-id', 'c-id'])
+        tree.commit('init', rev_id='one')
+        self.tree = tree
+
+    def assertInventoryEqual(self, expected, *args, **kwargs):
+        """Test that the output of 'bzr inventory' is as expected.
+
+        Any arguments supplied will be passed to run_bzr.
+        """
+        out, err = self.run_bzr('inventory', *args, **kwargs)
+        self.assertEqual(expected, out)
+        self.assertEqual('', err)
+        
+    def test_inventory(self):
+        self.assertInventoryEqual('a\nb\nb/c\n')
+
+    def test_inventory_kind(self):
+        self.assertInventoryEqual('a\nb/c\n', '--kind', 'file')
+        self.assertInventoryEqual('b\n', '--kind', 'directory')
+
+    def test_inventory_show_ids(self):
+        expected = ''.join(('%-50s %s\n' % (path, file_id))
+                                for path, file_id in
+                                [('a', 'a-id'),
+                                 ('b', 'b-id'),
+                                 ('b/c', 'c-id')
+                                ]
+                          )
+        self.assertInventoryEqual(expected, '--show-ids')
+
+    def test_inventory_specific_files(self):
+        self.assertInventoryEqual('a\n', 'a')
+        self.assertInventoryEqual('b\nb/c\n', 'b', 'b/c')
+        # 'bzr inventory' recurses into subdirectories
+        self.assertInventoryEqual('b\nb/c\n', 'b')
+
+    def test_inventory_mixed(self):
+        """Test that we get expected results when mixing parameters"""
+        a_line = '%-50s %s\n' % ('a', 'a-id')
+        b_line = '%-50s %s\n' % ('b', 'b-id')
+        c_line = '%-50s %s\n' % ('b/c', 'c-id')
+
+        self.assertInventoryEqual('', '--kind', 'directory', 'a')
+        self.assertInventoryEqual(a_line + c_line, '--kind', 'file',
+                                                   '--show-ids')
+        self.assertInventoryEqual(c_line, '--kind', 'file', '--show-ids',
+                                          'b', 'b/c')
+
+    def test_in_subdir(self):
+        os.chdir('b')
+        # TODO: jam 20060922 Maybe inventory should return the paths as
+        #       relative to '.', rather than relative to root
+
+        # a plain 'inventory' returns all files
+        self.assertInventoryEqual('a\nb\nb/c\n')
+        # But passing '.' will only return paths underneath here
+        self.assertInventoryEqual('b\nb/c\n', '.')
+
+
+    def test_inventory_revision(self):
+        self.build_tree(['b/d', 'e'])
+        self.tree.add(['b/d', 'e'], ['d-id', 'e-id'])
+        self.tree.commit('add files')
+
+        self.tree.rename_one('b/d', 'd')
+        self.tree.commit('rename b/d => d')
+
+        # Passing just -r returns the inventory of that revision
+        self.assertInventoryEqual('a\nb\nb/c\n', '-r', '1')
+        self.assertInventoryEqual('a\nb\nb/c\nb/d\ne\n', '-r', '2')
+
+        # Passing a path will lookup the path in the old and current locations
+        self.assertInventoryEqual('b/d\n', '-r', '2', 'b/d')
+        self.assertInventoryEqual('b/d\n', '-r', '2', 'd')
+
+        self.tree.rename_one('e', 'b/e')
+        self.tree.commit('rename e => b/e')
+
+        # When supplying just a directory paths that are now,
+        # or used to be, in that directory are shown
+        self.assertInventoryEqual('b\nb/c\nb/d\ne\n', '-r', '2', 'b')
+
+    def test_missing_file(self):
+        self.run_bzr_error([r'Path\(s\) are not versioned: no-such-file'],
+                           'inventory', 'no-such-file')

=== modified file 'bzrlib/tests/blackbox/test_selftest.py'
--- bzrlib/tests/blackbox/test_selftest.py	2006-09-15 02:04:56 +0000
+++ bzrlib/tests/blackbox/test_selftest.py	2006-09-25 19:21:12 +0000
@@ -86,8 +86,53 @@
 
 class TestRunBzr(ExternalBase):
 
-    def run_bzr_captured(self, argv, retcode=0, encoding=None, stdin=None):
+    def run_bzr_captured(self, argv, retcode=0, encoding=None, stdin=None,
+                         working_dir=None):
+        """Override run_bzr_captured to test how it is invoked by run_bzr.
+
+        We test how run_bzr_captured actually invokes bzr in another location.
+        Here we only need to test that it is run_bzr passes the right
+        parameters to run_bzr_captured.
+        """
+        self.argv = argv
+        self.retcode = retcode
+        self.encoding = encoding
         self.stdin = stdin
+        self.working_dir = working_dir
+
+    def test_args(self):
+        """Test that run_bzr passes args correctly to run_bzr_captured"""
+        self.run_bzr('arg1', 'arg2', 'arg3', retcode=1)
+        self.assertEqual(('arg1', 'arg2', 'arg3'), self.argv)
+
+    def test_encoding(self):
+        """Test that run_bzr passes encoding to run_bzr_captured"""
+        self.run_bzr('foo', 'bar')
+        self.assertEqual(None, self.encoding)
+        self.assertEqual(('foo', 'bar'), self.argv)
+
+        self.run_bzr('foo', 'bar', encoding='baz')
+        self.assertEqual('baz', self.encoding)
+        self.assertEqual(('foo', 'bar'), self.argv)
+
+    def test_retcode(self):
+        """Test that run_bzr passes retcode to run_bzr_captured"""
+        # Default is retcode == 0
+        self.run_bzr('foo', 'bar')
+        self.assertEqual(0, self.retcode)
+        self.assertEqual(('foo', 'bar'), self.argv)
+
+        self.run_bzr('foo', 'bar', retcode=1)
+        self.assertEqual(1, self.retcode)
+        self.assertEqual(('foo', 'bar'), self.argv)
+
+        self.run_bzr('foo', 'bar', retcode=None)
+        self.assertEqual(None, self.retcode)
+        self.assertEqual(('foo', 'bar'), self.argv)
+
+        self.run_bzr('foo', 'bar', retcode=3)
+        self.assertEqual(3, self.retcode)
+        self.assertEqual(('foo', 'bar'), self.argv)
 
     def test_stdin(self):
         # test that the stdin keyword to run_bzr is passed through to
@@ -97,8 +142,21 @@
         # should invoke it.
         self.run_bzr('foo', 'bar', stdin='gam')
         self.assertEqual('gam', self.stdin)
+        self.assertEqual(('foo', 'bar'), self.argv)
+
         self.run_bzr('foo', 'bar', stdin='zippy')
         self.assertEqual('zippy', self.stdin)
+        self.assertEqual(('foo', 'bar'), self.argv)
+
+    def test_working_dir(self):
+        """Test that run_bzr passes working_dir to run_bzr_captured"""
+        self.run_bzr('foo', 'bar')
+        self.assertEqual(None, self.working_dir)
+        self.assertEqual(('foo', 'bar'), self.argv)
+
+        self.run_bzr('foo', 'bar', working_dir='baz')
+        self.assertEqual('baz', self.working_dir)
+        self.assertEqual(('foo', 'bar'), self.argv)
 
 
 class TestBenchmarkTests(TestCaseWithTransport):
@@ -133,6 +191,7 @@
         self.stdin = stdin
         self.factory_stdin = getattr(bzrlib.ui.ui_factory, "stdin", None)
         self.factory = bzrlib.ui.ui_factory
+        self.working_dir = osutils.getcwd()
         stdout.write('foo\n')
         stderr.write('bar\n')
         return 0
@@ -163,6 +222,32 @@
         self.assertEqual('bar\n', self.factory.stderr.getvalue())
         self.assertIsInstance(self.factory, bzrlib.tests.blackbox.TestUIFactory)
 
+    def test_working_dir(self):
+        self.build_tree(['one/', 'two/'])
+        cwd = osutils.getcwd()
+
+        # Default is to work in the current directory
+        self.run_bzr_captured(['foo', 'bar'])
+        self.assertEqual(cwd, self.working_dir)
+
+        self.run_bzr_captured(['foo', 'bar'], working_dir=None)
+        self.assertEqual(cwd, self.working_dir)
+
+        # The function should be run in the alternative directory
+        # but afterwards the current working dir shouldn't be changed
+        self.run_bzr_captured(['foo', 'bar'], working_dir='one')
+        self.assertNotEqual(cwd, self.working_dir)
+        self.assertEndsWith(self.working_dir, 'one')
+        self.assertEqual(cwd, osutils.getcwd())
+
+        self.run_bzr_captured(['foo', 'bar'], working_dir='two')
+        self.assertNotEqual(cwd, self.working_dir)
+        self.assertEndsWith(self.working_dir, 'two')
+        self.assertEqual(cwd, osutils.getcwd())
+
+
+class TestRunBzrSubprocess(TestCaseWithTransport):
+
     def test_run_bzr_subprocess(self):
         """The run_bzr_helper_external comand behaves nicely."""
         result = self.run_bzr_subprocess('--version')
@@ -244,6 +329,38 @@
         self.assertEqual('it sure does!\n', out)
         self.assertEqual('', err)
 
+    def test_run_bzr_subprocess_working_dir(self):
+        """Test that we can specify the working dir for the child"""
+        cwd = osutils.getcwd()
+
+        self.make_branch_and_tree('.')
+        self.make_branch_and_tree('one')
+        self.make_branch_and_tree('two')
+
+        def get_root(**kwargs):
+            """Spawn a process to get the 'root' of the tree.
+
+            You can pass in arbitrary new arguments. This just makes
+            sure that the returned path doesn't have trailing whitespace.
+            """
+            return self.run_bzr_subprocess('root', **kwargs)[0].rstrip()
+
+        self.assertEqual(cwd, get_root())
+        self.assertEqual(cwd, get_root(working_dir=None))
+        # Has our path changed?
+        self.assertEqual(cwd, osutils.getcwd())
+
+        dir1 = get_root(working_dir='one')
+        self.assertEndsWith(dir1, 'one')
+        self.assertEqual(cwd, osutils.getcwd())
+
+        dir2 = get_root(working_dir='two')
+        self.assertEndsWith(dir2, 'two')
+        self.assertEqual(cwd, osutils.getcwd())
+
+
+class TestBzrSubprocess(TestCaseWithTransport):
+
     def test_start_and_stop_bzr_subprocess(self):
         """We can start and perform other test actions while that process is
         still alive.
@@ -287,6 +404,16 @@
                                             retcode=3)
         self.assertEqual('', result[0])
         self.assertEqual('bzr: interrupted\n', result[1])
+
+    def test_start_and_stop_working_dir(self):
+        cwd = osutils.getcwd()
+
+        self.make_branch_and_tree('one')
+
+        process = self.start_bzr_subprocess(['root'], working_dir='one')
+        result = self.finish_bzr_subprocess(process)
+        self.assertEndsWith(result[0], 'one\n')
+        self.assertEqual('', result[1])
         
 
 class TestRunBzrError(ExternalBase):

=== modified file 'bzrlib/tests/blackbox/test_too_much.py'
--- bzrlib/tests/blackbox/test_too_much.py	2006-09-20 23:43:18 +0000
+++ bzrlib/tests/blackbox/test_too_much.py	2006-09-21 20:55:45 +0000
@@ -147,22 +147,6 @@
         test.runbzr('add goodbye')
         test.runbzr('commit -m setup goodbye')
 
-    def test_inventory(self):
-        bzr = self.runbzr
-        def output_equals(value, *args):
-            out = self.runbzr(['inventory'] + list(args), backtick=True)
-            self.assertEquals(out, value)
-
-        bzr('init')
-        open('a', 'wb').write('hello\n')
-        os.mkdir('b')
-
-        bzr('add a b')
-        bzr('commit -m add')
-
-        output_equals('a\n', '--kind', 'file')
-        output_equals('b\n', '--kind', 'directory')
-
     def test_pull_verbose(self):
         """Pull changes from one branch to another and watch the output."""
 

=== modified file 'bzrlib/tests/blackbox/test_version_info.py'
--- bzrlib/tests/blackbox/test_version_info.py	2006-09-21 16:48:22 +0000
+++ bzrlib/tests/blackbox/test_version_info.py	2006-09-22 15:19:55 +0000
@@ -26,7 +26,7 @@
     def test_invalid_format(self):
         self.run_bzr('version-info', '--format', 'quijibo', retcode=3)
 
-    def create_branch(self):
+    def create_tree(self):
         wt = self.make_branch_and_tree('branch')
 
         self.build_tree(['branch/a'])
@@ -38,9 +38,10 @@
         wt.commit('adding b', rev_id='r2')
 
         self.revisions = wt.branch.revision_history()
+        return wt
 
     def test_basic(self):
-        self.create_branch()
+        self.create_tree()
 
         txt = self.run_bzr('version-info', 'branch')[0]
         self.assertContainsRe(txt, 'date:')
@@ -50,7 +51,7 @@
 
     def test_all(self):
         """'--all' includes clean, revision history, and file revisions"""
-        self.create_branch()
+        self.create_tree()
         txt = self.run_bzr('version-info', 'branch',
                            '--all')[0]
         self.assertContainsRe(txt, 'date:')
@@ -68,7 +69,7 @@
 
     def test_clean(self):
         """Test that --check-clean includes the right info"""
-        self.create_branch()
+        self.create_tree()
 
         txt = self.run_bzr('version-info', 'branch',
                            '--check-clean')[0]
@@ -85,29 +86,53 @@
 
         os.remove('branch/c')
 
+    def test_no_working_tree(self):
+        tree = self.create_tree()
+        branch = self.make_branch('just_branch')
+        branch.pull(tree.branch)
+
+        txt = self.run_bzr('version-info', 'just_branch')[0]
+        self.assertStartsWith(txt, 'revision-id: r2\n')
+
+    def assertEqualNoBuildDate(self, text1, text2):
+        """Compare 2 texts, but ignore the build-date field.
+
+        build-date is the current timestamp, accurate to seconds. But the
+        clock is always ticking, and it may have ticked between the time
+        that text1 and text2 were generated.
+        """
+        lines1 = text1.splitlines(True)
+        lines2 = text2.splitlines(True)
+        for line1, line2 in zip(lines1, lines2):
+            if line1.startswith('build-date: '):
+                self.assertStartsWith(line2, 'build-date: ')
+            else:
+                self.assertEqual(line1, line2)
+        self.assertEqual(len(lines1), len(lines2))
+
     def test_no_branch(self):
         """Test that bzr defaults to the local working directory"""
-        self.create_branch()
+        self.create_tree()
 
         txt1 = self.run_bzr('version-info', 'branch')[0]
 
         os.chdir('branch')
         txt2 = self.run_bzr('version-info')[0]
-        self.assertEqual(txt1, txt2)
+        self.assertEqualNoBuildDate(txt1, txt2)
 
     def test_rio(self):
         """Test that we can pass --format=rio"""
-        self.create_branch()
+        self.create_tree()
 
         txt = self.run_bzr('version-info', 'branch')[0]
         txt1 = self.run_bzr('version-info', '--format', 'rio', 'branch')[0]
         txt2 = self.run_bzr('version-info', '--format=rio', 'branch')[0]
-        self.assertEqual(txt, txt1)
-        self.assertEqual(txt, txt2)
+        self.assertEqualNoBuildDate(txt, txt1)
+        self.assertEqualNoBuildDate(txt, txt2)
 
     def test_python(self):
         """Test that we can do --format=python"""
-        self.create_branch()
+        self.create_tree()
 
         txt = self.run_bzr('version-info', '--format', 'python', 'branch')[0]
 

=== modified file 'bzrlib/tests/repository_implementations/test_commit_builder.py'
--- bzrlib/tests/repository_implementations/test_commit_builder.py	2006-09-10 20:12:00 +0000
+++ bzrlib/tests/repository_implementations/test_commit_builder.py	2006-09-27 02:28:44 +0000
@@ -95,3 +95,15 @@
         # precisely test that - a repository that wants to can add it on deserialisation,
         # but thats all the current contract guarantees anyway.
         self.assertEqual(rev_id, tree.branch.repository.get_inventory(rev_id).revision_id)
+
+    def test_revision_tree(self):
+        tree = self.make_branch_and_tree(".")
+        builder = tree.branch.get_commit_builder([])
+        self.record_root(builder, tree)
+        builder.finish_inventory()
+        rev_id = builder.commit('foo bar')
+        rev_tree = builder.revision_tree()
+        # Just a couple simple tests to ensure that it actually follows
+        # the RevisionTree api.
+        self.assertEqual(rev_id, rev_tree.get_revision_id())
+        self.assertEqual([], rev_tree.get_parent_ids())

=== modified file 'bzrlib/tests/test_command.py'
--- bzrlib/tests/test_command.py	2006-08-22 21:31:23 +0000
+++ bzrlib/tests/test_command.py	2006-09-01 03:55:07 +0000
@@ -21,7 +21,7 @@
     errors,
     )
 from bzrlib.commands import display_command
-from bzrlib.tests import TestCase
+from bzrlib.tests import TestCase, TestSkipped
 
 
 class TestCommands(TestCase):
@@ -49,6 +49,9 @@
     def test_unicode_option(self):
         # This error is actually thrown by optparse, when it
         # can't find the given option
+        import optparse
+        if optparse.__version__ == "1.5.3":
+            raise TestSkipped("optparse 1.5.3 can't handle unicode options")
         self.assertRaises(errors.BzrCommandError,
                           commands.run_bzr, ['log', u'--option\xb5'])
 

=== modified file 'bzrlib/tests/test_commit.py'
--- bzrlib/tests/test_commit.py	2006-09-04 08:00:00 +0000
+++ bzrlib/tests/test_commit.py	2006-09-22 18:49:04 +0000
@@ -18,6 +18,7 @@
 import os
 
 import bzrlib
+from bzrlib import errors
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir, BzrDirMetaFormat1
@@ -543,3 +544,9 @@
         timestamp = rev.timestamp
         timestamp_1ms = round(timestamp, 3)
         self.assertEqual(timestamp_1ms, timestamp)
+
+    def test_commit_unversioned_specified(self):
+        """Commit should raise if specified files isn't in basis or worktree"""
+        tree = self.make_branch_and_tree('.')
+        self.assertRaises(errors.PathsNotVersionedError, tree.commit, 
+                          'message', specific_files=['bogus'])

=== modified file 'bzrlib/tests/test_plugins.py'
--- bzrlib/tests/test_plugins.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/test_plugins.py	2006-09-25 21:54:28 +0000
@@ -148,8 +148,8 @@
         help = {}
         current = None
         for line in self.capture('help commands').splitlines():
-            if line.startswith('bzr '):
-                current = line.split()[1]
+            if not line.startswith(' '):
+                current = line.split()[0]
             help[current] = help.get(current, '') + line
 
         return help
@@ -189,7 +189,7 @@
             help = self.capture('help myplug')
             self.assertContainsRe(help, 'From plugin "myplug"')
             help = self.split_help_commands()['myplug']
-            self.assertContainsRe(help, 'From plugin "myplug"')
+            self.assertContainsRe(help, '\[myplug\]')
         finally:
             # remove the plugin 'plugin'
             if getattr(bzrlib.plugins, 'plugin', None):

=== modified file 'bzrlib/tests/test_rio.py'
--- bzrlib/tests/test_rio.py	2006-03-09 18:47:40 +0000
+++ bzrlib/tests/test_rio.py	2006-09-22 19:37:57 +0000
@@ -27,6 +27,9 @@
 import sys
 from tempfile import TemporaryFile
 
+from bzrlib import (
+    rio,
+    )
 from bzrlib.tests import TestCaseInTempDir, TestCase
 from bzrlib.rio import (RioWriter, Stanza, read_stanza, read_stanzas, rio_file,
                         RioReader)
@@ -320,8 +323,6 @@
         self.assertRaises(TypeError, s.add, 10, {})
 
     def test_rio_unicode(self):
-        # intentionally use cStringIO which doesn't accomodate unencoded unicode objects
-        sio = cStringIO.StringIO()
         uni_data = u'\N{KATAKANA LETTER O}'
         s = Stanza(foo=uni_data)
         self.assertEquals(s.get('foo'), uni_data)
@@ -331,3 +332,24 @@
         new_s = read_stanza(raw_lines)
         self.assertEquals(new_s.get('foo'), uni_data)
 
+    def test_rio_to_unicode(self):
+        uni_data = u'\N{KATAKANA LETTER O}'
+        s = Stanza(foo=uni_data)
+        unicode_str = s.to_unicode()
+        self.assertEqual(u'foo: %s\n' % (uni_data,), unicode_str)
+        new_s = rio.read_stanza_unicode(unicode_str.splitlines(True))
+        self.assertEqual(uni_data, new_s.get('foo'))
+
+    def test_nested_rio_unicode(self):
+        uni_data = u'\N{KATAKANA LETTER O}'
+        s = Stanza(foo=uni_data)
+        parent_stanza = Stanza(child=s.to_unicode())
+        raw_lines = parent_stanza.to_lines()
+        self.assertEqual(['child: foo: ' + uni_data.encode('utf-8') + '\n',
+                          '\t\n',
+                         ], raw_lines)
+        new_parent = read_stanza(raw_lines)
+        child_text = new_parent.get('child')
+        self.assertEqual(u'foo: %s\n' % uni_data, child_text)
+        new_child = rio.read_stanza_unicode(child_text.splitlines(True))
+        self.assertEqual(uni_data, new_child.get('foo'))

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2006-09-21 16:03:07 +0000
+++ bzrlib/tests/test_selftest.py	2006-09-24 18:42:09 +0000
@@ -15,6 +15,7 @@
 
 """Tests for the test framework."""
 
+import cStringIO
 import os
 from StringIO import StringIO
 import sys
@@ -750,6 +751,68 @@
             revision_id = workingtree.get_parent_ids()[0]
             self.assertEndsWith(output_string.rstrip(), revision_id)
 
+    def test_success_log_deleted(self):
+        """Successful tests have their log deleted"""
+
+        class LogTester(TestCase):
+
+            def test_success(self):
+                self.log('this will be removed\n')
+
+        sio = cStringIO.StringIO()
+        runner = TextTestRunner(stream=sio)
+        test = LogTester('test_success')
+        result = self.run_test_runner(runner, test)
+
+        log = test._get_log()
+        self.assertEqual("DELETED log file to reduce memory footprint", log)
+        self.assertEqual('', test._log_contents)
+        self.assertIs(None, test._log_file_name)
+
+    def test_fail_log_kept(self):
+        """Failed tests have their log kept"""
+
+        class LogTester(TestCase):
+
+            def test_fail(self):
+                self.log('this will be kept\n')
+                self.fail('this test fails')
+
+        sio = cStringIO.StringIO()
+        runner = TextTestRunner(stream=sio)
+        test = LogTester('test_fail')
+        result = self.run_test_runner(runner, test)
+
+        text = sio.getvalue()
+        self.assertContainsRe(text, 'this will be kept')
+        self.assertContainsRe(text, 'this test fails')
+
+        log = test._get_log()
+        self.assertContainsRe(log, 'this will be kept')
+        self.assertEqual(log, test._log_contents)
+
+    def test_error_log_kept(self):
+        """Tests with errors have their log kept"""
+
+        class LogTester(TestCase):
+
+            def test_error(self):
+                self.log('this will be kept\n')
+                raise ValueError('random exception raised')
+
+        sio = cStringIO.StringIO()
+        runner = TextTestRunner(stream=sio)
+        test = LogTester('test_error')
+        result = self.run_test_runner(runner, test)
+
+        text = sio.getvalue()
+        self.assertContainsRe(text, 'this will be kept')
+        self.assertContainsRe(text, 'random exception raised')
+
+        log = test._get_log()
+        self.assertContainsRe(log, 'this will be kept')
+        self.assertEqual(log, test._log_contents)
+
 
 class TestTestCase(TestCase):
     """Tests that test the core bzrlib TestCase."""

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2006-09-22 04:52:17 +0000
+++ bzrlib/tests/test_transform.py	2006-09-27 02:28:44 +0000
@@ -541,6 +541,21 @@
         self.assertTrue(wt.is_executable('soc'))
         self.assertTrue(wt.is_executable('sac'))
 
+    def test_preserve_mode(self):
+        """File mode is preserved when replacing content"""
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+        transform, root = self.get_transform()
+        transform.new_file('file1', root, 'contents', 'file1-id', True)
+        transform.apply()
+        self.assertTrue(self.wt.is_executable('file1-id'))
+        transform, root = self.get_transform()
+        file1_id = transform.trans_id_tree_file_id('file1-id')
+        transform.delete_contents(file1_id)
+        transform.create_file('contents2', file1_id)
+        transform.apply()
+        self.assertTrue(self.wt.is_executable('file1-id'))
+
     def test__set_mode_stats_correctly(self):
         """_set_mode stats to determine file mode."""
         if sys.platform == 'win32':

=== modified file 'bzrlib/tests/test_version_info.py'
--- bzrlib/tests/test_version_info.py	2006-09-22 04:52:17 +0000
+++ bzrlib/tests/test_version_info.py	2006-09-27 02:28:44 +0000
@@ -42,7 +42,7 @@
         wt.commit('b', rev_id='r2')
 
         self.build_tree_contents([('branch/a', 'new contents\n')])
-        wt.commit('a2', rev_id='r3')
+        wt.commit(u'\xe52', rev_id='r3')
 
         return wt
 
@@ -77,7 +77,7 @@
         self.assertContainsRe(val, 'id: r2')
         self.assertContainsRe(val, 'message: b')
         self.assertContainsRe(val, 'id: r3')
-        self.assertContainsRe(val, 'message: a2')
+        self.assertContainsRe(val, 'message: \xc3\xa52') # utf8 encoding '\xe5'
 
     def test_rio_version(self):
         wt = self.create_branch()
@@ -120,7 +120,7 @@
         stanza = regen(include_revision_history=True)
         revision_stanza = get_one_stanza(stanza, 'revisions')
         self.assertEqual(['r1', 'r2', 'r3'], revision_stanza.get_all('id'))
-        self.assertEqual(['a', 'b', 'a2'], revision_stanza.get_all('message'))
+        self.assertEqual(['a', 'b', u'\xe52'], revision_stanza.get_all('message'))
         self.assertEqual(3, len(revision_stanza.get_all('date')))
 
         # a was modified, so it should show up modified again
@@ -195,7 +195,7 @@
 
         rev_info = [(rev, message) for rev, message, timestamp, timezone
                                    in tvi.revisions]
-        self.assertEqual([('r1', 'a'), ('r2', 'b'), ('r3', 'a2')], rev_info)
+        self.assertEqual([('r1', 'a'), ('r2', 'b'), ('r3', u'\xe52')], rev_info)
 
         # a was modified, so it should show up modified again
         self.build_tree(['branch/a', 'branch/c'])

=== modified file 'bzrlib/tests/test_versionedfile.py'
--- bzrlib/tests/test_versionedfile.py	2006-07-28 16:05:23 +0000
+++ bzrlib/tests/test_versionedfile.py	2006-09-25 18:30:50 +0000
@@ -24,7 +24,10 @@
 from StringIO import StringIO
 
 import bzrlib
-import bzrlib.errors as errors
+from bzrlib import (
+    errors,
+    progress,
+    )
 from bzrlib.errors import (
                            RevisionNotPresent, 
                            RevisionAlreadyPresent,
@@ -543,6 +546,17 @@
         # versions in the weave 
         # the ordering here is to make a tree so that dumb searches have
         # more changes to muck up.
+
+        class InstrumentedProgress(progress.DummyProgress):
+
+            def __init__(self):
+
+                progress.DummyProgress.__init__(self)
+                self.updates = []
+
+            def update(self, msg=None, current=None, total=None):
+                self.updates.append((msg, current, total))
+
         vf = self.get_file()
         # add a base to get included
         vf.add_lines('base', [], ['base\n'])
@@ -556,7 +570,7 @@
         vf.add_lines('otherchild',
                      ['lancestor', 'base'],
                      ['base\n', 'lancestor\n', 'otherchild\n'])
-        def iter_with_versions(versions):
+        def iter_with_versions(versions, expected):
             # now we need to see what lines are returned, and how often.
             lines = {'base\n':0,
                      'lancestor\n':0,
@@ -564,18 +578,32 @@
                      'child\n':0,
                      'otherchild\n':0,
                      }
+            progress = InstrumentedProgress()
             # iterate over the lines
-            for line in vf.iter_lines_added_or_present_in_versions(versions):
+            for line in vf.iter_lines_added_or_present_in_versions(versions, 
+                pb=progress):
                 lines[line] += 1
+            if []!= progress.updates: 
+                self.assertEqual(expected, progress.updates)
             return lines
-        lines = iter_with_versions(['child', 'otherchild'])
+        lines = iter_with_versions(['child', 'otherchild'], 
+                                   [('Walking content.', 0, 2), 
+                                    ('Walking content.', 0, 2), 
+                                    ('Walking content.', 3, 2), 
+                                    ('Walking content.', 2, 2)])
         # we must see child and otherchild
         self.assertTrue(lines['child\n'] > 0)
         self.assertTrue(lines['otherchild\n'] > 0)
         # we dont care if we got more than that.
         
         # test all lines
-        lines = iter_with_versions(None)
+        lines = iter_with_versions(None, [('Walking content.', 0, 5), 
+                                          ('Walking content.', 0, 5), 
+                                          ('Walking content.', 1, 5), 
+                                          ('Walking content.', 2, 5), 
+                                          ('Walking content.', 2, 5), 
+                                          ('Walking content.', 3, 5), 
+                                          ('Walking content.', 5, 5)])
         # all lines must be seen at least once
         self.assertTrue(lines['base\n'] > 0)
         self.assertTrue(lines['lancestor\n'] > 0)

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2006-09-08 18:46:29 +0000
+++ bzrlib/trace.py	2006-09-22 03:02:40 +0000
@@ -54,6 +54,7 @@
 import errno
 import os
 import sys
+import re
 import logging
 
 import bzrlib
@@ -168,9 +169,6 @@
     """
     if msg:
         error(msg)
-    else:
-        exc_str = format_exception_short(sys.exc_info())
-        error(exc_str)
     log_exception_quietly()
 
 
@@ -288,7 +286,8 @@
     """Report an exception that probably indicates a bug in bzr"""
     import traceback
     exc_type, exc_object, exc_tb = exc_info
-    print >>err_file, "bzr: ERROR: %s: %s" % (exc_type, exc_object)
+    print >>err_file, "bzr: ERROR: %s.%s: %s" % (
+        exc_type.__module__, exc_type.__name__, exc_object)
     print >>err_file
     traceback.print_exception(exc_type, exc_object, exc_tb, file=err_file)
     print >>err_file

=== modified file 'bzrlib/transport/smart.py'
--- bzrlib/transport/smart.py	2006-09-18 09:58:40 +0000
+++ bzrlib/transport/smart.py	2006-09-26 05:51:37 +0000
@@ -518,6 +518,9 @@
 
     def accept_and_serve(self):
         conn, client_addr = self._server_socket.accept()
+        # For WIN32, where the timeout value from the listening socket
+        # propogates to the newly accepted socket.
+        conn.setblocking(True)
         conn.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
         from_client = conn.makefile('r')
         to_client = conn.makefile('w')

=== modified file 'bzrlib/version_info_formats/format_rio.py'
--- bzrlib/version_info_formats/format_rio.py	2006-09-20 15:11:19 +0000
+++ bzrlib/version_info_formats/format_rio.py	2006-09-22 05:38:28 +0000
@@ -16,8 +16,6 @@
 
 """A generator which creates a rio stanza of the current tree info"""
 
-from cStringIO import StringIO
-
 from bzrlib.rio import RioWriter, Stanza
 
 from bzrlib.version_info_formats import (
@@ -63,20 +61,14 @@
                 log.add('id', revision_id)
                 log.add('message', message)
                 log.add('date', create_date_str(timestamp, timezone))
-            sio = StringIO()
-            log_writer = RioWriter(to_file=sio)
-            log_writer.write_stanza(log)
-            info.add('revisions', sio.getvalue())
+            info.add('revisions', log.to_unicode())
 
         if self._include_file_revs:
             files = Stanza()
             for path in sorted(self._file_revisions.keys()):
                 files.add('path', path)
                 files.add('revision', self._file_revisions[path])
-            sio = StringIO()
-            file_writer = RioWriter(to_file=sio)
-            file_writer.write_stanza(files)
-            info.add('file-revisions', sio.getvalue())
+            info.add('file-revisions', files.to_unicode())
 
         writer = RioWriter(to_file=to_file)
         writer.write_stanza(info)

=== modified file 'bzrlib/versionedfile.py'
--- bzrlib/versionedfile.py	2006-08-22 16:14:17 +0000
+++ bzrlib/versionedfile.py	2006-09-25 14:52:38 +0000
@@ -403,7 +403,8 @@
             version_ids,
             ignore_missing)
 
-    def iter_lines_added_or_present_in_versions(self, version_ids=None):
+    def iter_lines_added_or_present_in_versions(self, version_ids=None, 
+                                                pb=None):
         """Iterate over the lines in the versioned file from version_ids.
 
         This may return lines from other versions, and does not return the
@@ -412,6 +413,10 @@
         thinks is relevant, but given that such hints are just guesses,
         its better not to have it if we don't need it.
 
+        If a progress bar is supplied, it may be used to indicate progress.
+        The caller is responsible for cleaning up progress bars (because this
+        is an iterator).
+
         NOTES: Lines are normalised: they will all have \n terminators.
                Lines are returned in arbitrary order.
         """

=== modified file 'bzrlib/weave.py'
--- bzrlib/weave.py	2006-09-08 18:52:17 +0000
+++ bzrlib/weave.py	2006-09-25 14:52:38 +0000
@@ -75,6 +75,9 @@
 import time
 import warnings
 
+from bzrlib import (
+    progress,
+    )
 from bzrlib.trace import mutter
 from bzrlib.errors import (WeaveError, WeaveFormatError, WeaveParentMismatch,
         RevisionAlreadyPresent,
@@ -664,7 +667,8 @@
         """_walk has become visit, a supported api."""
         return self._walk_internal()
 
-    def iter_lines_added_or_present_in_versions(self, version_ids=None):
+    def iter_lines_added_or_present_in_versions(self, version_ids=None,
+                                                pb=None):
         """See VersionedFile.iter_lines_added_or_present_in_versions()."""
         if version_ids is None:
             version_ids = self.versions()

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-09-22 04:52:17 +0000
+++ bzrlib/workingtree.py	2006-09-27 02:28:44 +0000
@@ -676,6 +676,22 @@
         """
         return self.get_parent_ids()[1:]
 
+    def _check_parents_for_ghosts(self, revision_ids, allow_leftmost_as_ghost):
+        """Common ghost checking functionality from set_parent_*.
+
+        This checks that the left hand-parent exists if there are any
+        revisions present.
+        """
+        if len(revision_ids) > 0:
+            leftmost_id = revision_ids[0]
+            if (not allow_leftmost_as_ghost and not
+                self.branch.repository.has_revision(leftmost_id)):
+                raise errors.GhostRevisionUnusableHere(leftmost_id)
+
+    def _set_merges_from_parent_ids(self, parent_ids):
+        merges = parent_ids[1:]
+        self._control_files.put_utf8('pending-merges', '\n'.join(merges))
+
     @needs_tree_write_lock
     def set_parent_ids(self, revision_ids, allow_leftmost_as_ghost=False):
         """Set the parent ids to revision_ids.
@@ -689,25 +705,42 @@
         :param revision_ids: The revision_ids to set as the parent ids of this
             working tree. Any of these may be ghosts.
         """
+        self._check_parents_for_ghosts(revision_ids,
+            allow_leftmost_as_ghost=allow_leftmost_as_ghost)
+
         if len(revision_ids) > 0:
-            leftmost_id = revision_ids[0]
-            if (not allow_leftmost_as_ghost and not
-                self.branch.repository.has_revision(leftmost_id)):
-                raise errors.GhostRevisionUnusableHere(leftmost_id)
-            self.set_last_revision(leftmost_id)
+            self.set_last_revision(revision_ids[0])
         else:
             self.set_last_revision(None)
-        merges = revision_ids[1:]
-        self._control_files.put_utf8('pending-merges', '\n'.join(merges))
+
+        self._set_merges_from_parent_ids(revision_ids)
 
     @needs_tree_write_lock
     def set_parent_trees(self, parents_list, allow_leftmost_as_ghost=False):
         """See MutableTree.set_parent_trees."""
-        # parent trees are not used in current format trees, delegate to
-        # set_parent_ids
-        self.set_parent_ids([rev for (rev, tree) in parents_list],
+        parent_ids = [rev for (rev, tree) in parents_list]
+
+        self._check_parents_for_ghosts(parent_ids,
             allow_leftmost_as_ghost=allow_leftmost_as_ghost)
 
+        if len(parent_ids) == 0:
+            leftmost_parent_id = None
+            leftmost_parent_tree = None
+        else:
+            leftmost_parent_id, leftmost_parent_tree = parents_list[0]
+
+        if self._change_last_revision(leftmost_parent_id):
+            if leftmost_parent_tree is None:
+                # If we don't have a tree, fall back to reading the
+                # parent tree from the repository.
+                self._cache_basis_inventory(leftmost_parent_id)
+            else:
+                inv = leftmost_parent_tree.inventory
+                xml = self._create_basis_xml_from_inventory(
+                                        leftmost_parent_id, inv)
+                self._write_basis_inventory(xml)
+        self._set_merges_from_parent_ids(parent_ids)
+
     @needs_tree_write_lock
     def set_pending_merges(self, rev_list):
         parents = self.get_parent_ids()
@@ -1389,6 +1422,18 @@
             self.branch.set_revision_history([new_revision])
         return True
 
+    def _write_basis_inventory(self, xml):
+        """Write the basis inventory XML to the basis-inventory file"""
+        assert isinstance(xml, str), 'serialised xml must be bytestring.'
+        path = self._basis_inventory_name()
+        sio = StringIO(xml)
+        self._control_files.put(path, sio)
+
+    def _create_basis_xml_from_inventory(self, revision_id, inventory):
+        """Create the text that will be saved in basis-inventory"""
+        inventory.revision_id = revision_id
+        return bzrlib.xml6.serializer_v6.write_inventory_to_string(inventory)
+
     def _cache_basis_inventory(self, new_revision):
         """Cache new_revision as the basis inventory."""
         # TODO: this should allow the ready-to-use inventory to be passed in,
@@ -1411,12 +1456,8 @@
                 'format="6"' not in firstline):
                 inv = self.branch.repository.deserialise_inventory(
                     new_revision, xml)
-                inv.revision_id = new_revision
-                xml = bzrlib.xml6.serializer_v6.write_inventory_to_string(inv)
-            assert isinstance(xml, str), 'serialised xml must be bytestring.'
-            path = self._basis_inventory_name()
-            sio = StringIO(xml)
-            self._control_files.put(path, sio)
+                xml = self._create_basis_xml_from_inventory(new_revision, inv)
+            self._write_basis_inventory(xml)
         except (errors.NoSuchRevision, errors.RevisionNotPresent):
             pass
 

=== modified file 'bzrlib/xml_serializer.py'
--- bzrlib/xml_serializer.py	2006-09-10 02:18:31 +0000
+++ bzrlib/xml_serializer.py	2006-09-22 00:51:36 +0000
@@ -28,6 +28,7 @@
     from cElementTree import (ElementTree, SubElement, Element,
                               XMLTreeBuilder, fromstring, tostring)
     import elementtree
+    ParseError = SyntaxError
 except ImportError:
     mutter('WARNING: using slower ElementTree; consider installing cElementTree'
            " and make sure it's on your PYTHONPATH")
@@ -35,6 +36,7 @@
                                               Element, XMLTreeBuilder,
                                               fromstring, tostring)
     import util.elementtree as elementtree
+    from xml.parsers.expat import ExpatError as ParseError
 
 from bzrlib import errors
 
@@ -52,13 +54,13 @@
     def read_inventory_from_string(self, xml_string):
         try:
             return self._unpack_inventory(fromstring(xml_string))
-        except SyntaxError, e:
+        except ParseError, e:
             raise errors.UnexpectedInventoryFormat(e)
 
     def read_inventory(self, f):
         try:
             return self._unpack_inventory(self._read_element(f))
-        except SyntaxError, e:
+        except ParseError, e:
             raise errors.UnexpectedInventoryFormat(e)
 
     def write_revision(self, rev, f):

